library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity opb_buttons is
generic
(
	C_BASEADDR		: std_logic_vector	:= X"00000000";
	C_HIGHADDR		: std_logic_vector	:= X"0000000F";
	C_OPB_AWIDTH	: integer				:= 32;
	C_OPB_DWIDTH	: integer				:= 32;
   C_CLK_FREQ		: integer				:= 125_000_000
);
port
(
	buttonPort		: in	std_logic_vector(2 downto 0);
	pressIRQ			: out	std_logic;

	OPB_Clk			: in	std_logic;
	OPB_Rst			: in	std_logic;
	Sl_DBus			: out	std_logic_vector(0 to C_OPB_DWIDTH-1);
	Sl_errAck		: out	std_logic;
	Sl_retry			: out	std_logic;
	Sl_toutSup		: out	std_logic;
	Sl_xferAck		: out	std_logic;
	OPB_ABus			: in	std_logic_vector(0 to C_OPB_AWIDTH-1);
	OPB_BE			: in	std_logic_vector(0 to C_OPB_DWIDTH/8-1);
	OPB_DBus			: in	std_logic_vector(0 to C_OPB_DWIDTH-1);
	OPB_RNW			: in	std_logic;
	OPB_select		: in	std_logic;
	OPB_seqAddr		: in	std_logic
);

	attribute SIGIS						: string;
	attribute SIGIS of OPB_Clk			: signal is "Clk";
	attribute SIGIS of OPB_Rst			: signal is "Rst";
	attribute SIGIS of pressIRQ		: signal is "INTR_LEVEL_HIGH";

end entity opb_buttons;

------------------------------------------------------------------------------
-- Architecture section
------------------------------------------------------------------------------

architecture IMP of opb_buttons is

	component ButtonFilter is
		generic(	CLK_FREQ			: integer;
					NUM_SWITCHES	: integer);
		port (	clk				: in	std_logic;
					reset				: in	std_logic;
					buttonArray		: in	std_logic_vector(NUM_SWITCHES-1 downto 0);
					pressArray		: out	std_logic_vector(NUM_SWITCHES-1 downto 0));
	end component;

	signal testReg				: std_logic_vector(31 downto 0);
	signal eventArray			: std_logic_vector(2 downto 0);
	signal pressArray			: std_logic_vector(2 downto 0);

	signal data_BUS2IP		: std_logic_vector(31 downto 0);
	signal data_IP2BUS		: std_logic_vector(31 downto 0);
	signal chipSel				: std_logic;
	signal subAddress			: std_logic_vector(0 to 3);

	function reverseVector (a: in std_logic_vector) return std_logic_vector is
		variable result: std_logic_vector(a'reverse_range);
	begin
		for i in a'range loop
			result((a'length-1)-i) := a(i);
 		end loop;
		return result;
	end;

begin

chipSel		<= '1' when (OPB_select='1' and OPB_ABus(0 to 25)=C_BASEADDR(0 to 25)) else '0';

Sl_DBus		<= reverseVector(data_IP2BUS);
data_BUS2IP	<= reverseVector(OPB_DBus);

Sl_errAck	<= '0';
Sl_retry		<= '0';
Sl_toutSup	<= '0';

subAddress	<= OPB_ABus(26 to 29);

Filter : ButtonFilter
generic map(CLK_FREQ			=> C_CLK_FREQ,
				NUM_SWITCHES	=> 3)
port map (	clk				=> OPB_Clk,
				reset				=> OPB_Rst,
				buttonArray		=> buttonPort,
				pressArray		=> pressArray);

ReadWriteControl : process(OPB_RNW, chipSel, subAddress, testReg, eventArray)
	variable data_IP2BUS_var	: std_logic_vector(31 downto 0);
	variable Sl_xferAck_var		: std_logic;
begin

	data_IP2BUS_var	:= (others => '0');
	Sl_xferAck_var		:= '0';
	
	-- Read
	if (OPB_RNW='1' and chipSel='1') then
		case subAddress is
			when x"0"	=> data_IP2BUS_var					:= testReg;
			when x"1"	=> data_IP2BUS_var(2 downto 0)	:= eventArray;
			when others	=> null;
		end case;
		Sl_xferAck_var	:= '1';
	end if;

	-- Write
	if (OPB_RNW='0' and chipSel='1') then
		Sl_xferAck_var	:= '1';
	end if;

	data_IP2BUS		<= data_IP2BUS_var;
	Sl_xferAck		<= Sl_xferAck_var;

end process ReadWriteControl;

pressIRQ <= '0' when (eventArray=b"000") else '1';

WriteControl : process(OPB_Clk)
begin
	if (OPB_Clk'event and OPB_Clk='1') then
		if (OPB_Rst='1') then
			testReg <= (others => '0');
			eventArray <= (others => '0');

		else
			eventArray <= pressArray or eventArray;

			if (OPB_RNW='0' and chipSel='1') then
				case subAddress is
					when x"0"	=> testReg		<= data_BUS2IP;
					when x"1"	=> eventArray	<= data_BUS2IP(2 downto 0);
					when others	=> null;
				end case;
			end if;

		end if;
	end if;
end process WriteControl;

end IMP;
