library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity Cell is
	port (	clk				: in	std_logic;
				reset				: in	std_logic;
					
				dIn				: in	std_logic_vector(15 downto 0);
				dOut				: out	std_logic;

				config			: in	std_logic_vector(11 downto 0);
				configWEn		: in	std_logic;

				failMode			: in	std_logic_vector(1 downto 0);
				failModeWEn		: in	std_logic);
end Cell;

architecture Arch of Cell is

	signal dInASelect		: std_logic_vector(3 downto 0);
	signal dInBSelect		: std_logic_vector(3 downto 0);
	signal lookUpTable	: std_logic_vector(3 downto 0);

	signal lutAddress		: std_logic_vector(1 downto 0);
	signal lutOut			: std_logic;

	signal failModeReg	: std_logic_vector(1 downto 0);

	function mux4to1(	input	: std_logic_vector(3 downto 0);
							sel	: std_logic_vector(1 downto 0)) return std_logic is
		variable result : std_logic;
	begin
		case sel is
			when b"00"	=> result := input(0);
			when b"01"	=> result := input(1);
			when b"10"	=> result := input(2);
			when b"11"	=> result := input(3);
			when others	=> result := '0';
		end case;
		return result;
	end function mux4to1;

	function mux16to1(	input	: std_logic_vector(15 downto 0);
								sel	: std_logic_vector(3 downto 0)) return std_logic is
		variable result : std_logic;
	begin
		case sel is
			when b"0000"	=> result := input(0);
			when b"0001"	=> result := input(1);
			when b"0010"	=> result := input(2);
			when b"0011"	=> result := input(3);
			when b"0100"	=> result := input(4);
			when b"0101"	=> result := input(5);
			when b"0110"	=> result := input(6);
			when b"0111"	=> result := input(7);
			when b"1000"	=> result := input(8);
			when b"1001"	=> result := input(9);
			when b"1010"	=> result := input(10);
			when b"1011"	=> result := input(11);
			when b"1100"	=> result := input(12);
			when b"1101"	=> result := input(13);
			when b"1110"	=> result := input(14);
			when b"1111"	=> result := input(15);
			when others		=> result := '0';
		end case;
		return result;
	end function mux16to1;

	function failSelect(	input	: std_logic;
								mode	: std_logic_vector(1 downto 0)) return std_logic is
		variable output : std_logic;
	begin
		case mode is
			when b"00"	=> output := input;
			when b"01"	=> output := not input;
			when b"10"	=> output := '0';
			when b"11"	=> output := '1';
			when others	=> output := '0';
		end case;
		return output;
	end function failSelect;

begin

lutAddress(1) <= mux16to1(dIn, dInBSelect);
lutAddress(0) <= mux16to1(dIn, dInASelect);

lutOut <= mux4to1(lookUpTable, lutAddress);

dOut	<= failSelect(lutOut, failModeReg);

-- ConfigWrite ------------------------------------------------------------------------------------------------
-- Set the Cells parameters
---------------------------------------------------------------------------------------------------------------
ConfigWrite : process(clk)
begin

	if (clk'event and clk='1') then

		if (reset='1') then
			dInBSelect	<= (others => '0');
			dInASelect	<= (others => '0');
			lookUpTable	<= (others => '0');
			failModeReg <= (others => '0');

		else

			if (configWEn='1') then
				dInBSelect	<= config(11 downto 8);
				dInASelect	<= config(7 downto 4);
				lookUpTable	<= config(3 downto 0);
			end if;

			if (failModeWEn='1') then
				failModeReg <= failMode;
			end if;

		end if;

	end if;

end process ConfigWrite;

end Arch;
